# def test_check_order_dict(self):
#     valid_order = 'test_order'
#     invalid_order = ''
#     invalid_order2 = None
#
#     self.assertTrue(ConfigurationChecker.check_order_dict(valid_order))
#
#     with self.assertRaises(NoValidOrder):
#         ConfigurationChecker.check_order_dict(invalid_order)
#     with self.assertRaises(NoValidOrder):
#         ConfigurationChecker.check_order_dict(invalid_order2)